import java.io.*;
import java.util.*;

import sv.kernel.*;

/** 
 * SVFileReader.java
 * "svb" file reader
 * @version 1.0   10/15/97
 * @author Byeongseob Ki
 * 
 */

public class SVFileReader {
  static String this_filename;

  public static long readFromInput(svserver svServer, 
                              DataInputStream in, int mode, String filename) {
     this_filename = filename;
     return readFromInput(svServer, in, mode);
  }

  /**
   * read "svb" file from the data file
   * @param in DataInputStream for the data file
   * @param mode if mode is 1, read from "open", else read feom "java SV"
   */
  public static long readFromInput(svserver svServer, 
                                       DataInputStream in, int mode) {
     int dim = -1;
     try {
       // read dimension
       dim  = in.readInt();   
     } catch (Exception e) {
       System.out.println("Unknown dimension ...");
       return -1;
     } 

     switch (dim) {
        case 1:
           return read_1d(svServer, in, mode);
        case 2:
           return read_2d(svServer, in, mode);
        case 3:
           return read_3d(svServer, in, mode);
        case 101:
           return read_con(svServer, in, mode);
        case 102:
           return read_vec(svServer, in, mode);
        case 103:
           return read_bar(svServer, in, mode);
        case 104:
           return read_tree(svServer, in, mode);
        default:
           System.out.println("File format error ...");
           return -1;
      }
  }

  /**
   * read ".svb" file from the data file and send those to Scivis by
   * "java SV filename"
   * @parm in DataInputStream for the data file
   * @parm out DataOutputStream for the socket
   */
  public static void sendFromInput(DataInputStream in, DataOutputStream out) {
     int dim = -1;
     try {
       // read dimension
       dim  = in.readInt();   
     } catch (Exception e) {
       System.out.println("Unknown dimension ...");
       return;
     } 

     switch (dim) {
        case 1:
           send_1d(in, out);
           break;
        case 2:
           send_2d(in, out);
           break;
        case 3:
           send_3d(in, out);
           break;
        case 101:                   
           send_con(in, out);
           break;
        case 102:                   
           send_vec(in, out);
           break;
        default:
           System.out.println("File format error ...");
           return;
      }
  }


  /**
   * read ".sva" file from the data file and send those to the server
   * @parm in BufferedReader for the data file
   * @parm out DataOutputStream for the socket
   */
  public static void sendFromInput(String filename,
                           BufferedReader in, DataOutputStream out) {
     int dim = -1;
     try {
       // read dimension
       String input  = in.readLine();   
       dim = Integer.valueOf(input).intValue();
     } catch (Exception e) {
       System.out.println("Unknown dimension ...");
       return;
     } 

     switch (dim) {
        case 1:
           sendAscii_1d(filename, in, out);
           break;
        case 2:
           sendAscii_2d(filename, in, out);
           break;
        case 3:
           sendAscii_3d(filename, in, out);
           break;
        case 101:
           sendAscii_con(filename, in, out);
           break;
        case 102:
           sendAscii_vec(filename, in, out);
           break;
        default:
           System.out.println("File format error ...");
           return;
      }
  }

  // read 1d file
  public static long read_1d(svserver svServer, DataInputStream in, int mode) {
     String title = null, title2 = null;
     String configs = null;
     String drawings = null;
     GFPlane gframe = null;
     CPlane cvalues = null;

     byte[] byteArr;
     int sz;

     long ret = -1;

     try {
        // read title 
        sz = in.readInt();
        byteArr = new byte[sz];
        in.read(byteArr, 0, sz);
        title = new String(byteArr);

        // read configuration stream
        sz = in.readInt();
        if (sz > 0) {
           byteArr = new byte[sz];
           in.read(byteArr, 0, sz);
           cvalues = new CPlane(new String(byteArr));
        } else
           cvalues = null;

        if (mode == 1) 
           cvalues = svServer.configHandler.config1d;
        else {
           CPlane cvalues2 = svServer.configHandler.config1d;
           cvalues2.xLabel = cvalues.xLabel;
           cvalues2.yLabel = cvalues.yLabel;
           cvalues = cvalues2;
        }
           

        // read drawing objects stream
        sz = in.readInt();
        if (sz > 0) {
           byteArr = new byte[sz];
           in.read(byteArr, 0, sz);
           drawings = new String(byteArr);
        } 

        // read the number of data sets
        int noOfDataSet = in.readInt(); 

        // now read the data sets
        for (int i=0; i<noOfDataSet; i++) {
           DataSet1D dataSet = null;
 
           sz = in.readInt();
           byteArr = new byte[sz];
           in.read(byteArr, 0, sz);
           title2 = new String(byteArr);
 
           int noOfTimeData = in.readInt(); 
           for (int k=0; k<noOfTimeData; k++) {
              TimeData1D readed = SerialMgr.parse1dStream(in);
 
              if (k == 0) 
                  dataSet = new DataSet1D(title2, readed);
              else 
                  dataSet.addTimeData(readed);
	   }
 
           if (i == 0) {
               long fid = svServer.fmanager.register(dataSet, -1, cvalues);
               gframe = (GFPlane)svServer.fmanager.getGFrame(fid);
               ret = fid;
           } else
               gframe.addDataSet(dataSet);
        } 
        gframe.createWhiteboard(drawings);
      
	///////////////////////////////////////////
        // NEED COLLABORATION
        //////////////////////////////////////////
        CollSender sender = new CollSender(svServer.cmanager, gframe);
        sender.start();
   
        in.close();
        return ret;

     } catch (Exception e) {
        System.out.println("Can't read the 1d data from a file ...");
        return -1;
     }     

  }

  // read 2d file
  public static long read_2d(svserver svServer, DataInputStream in, int mode) {
     String title = null, title2 = null;
     String configs = null;
     String drawings = null;
     GFrame2D gframe = null;
     Config2D cvalues = null;

     byte[] byteArr;
     int sz;

     long ret = -1;

     try {
        // read title 
        sz = in.readInt();
        byteArr = new byte[sz];
        in.read(byteArr, 0, sz);
        title = new String(byteArr);

        // read configuration stream
        sz = in.readInt();
        if (sz > 0) {
           byteArr = new byte[sz];
           in.read(byteArr, 0, sz);
           cvalues = new Config2D(new String(byteArr));
        } else 
           cvalues = null;

        if (mode == 1) 
           cvalues = svServer.configHandler.config2d;
        else {
           Config2D cvalues2 = svServer.configHandler.config2d;
           cvalues2.xLabel = cvalues.xLabel;
           cvalues2.yLabel = cvalues.yLabel;
           cvalues2.zLabel = cvalues.zLabel;
           cvalues = cvalues2;
        }


        // read drawing objects stream
        sz = in.readInt();    // FLUSH OUT SINCE NOT IMPLEMENTED

        // read the number of data sets
        int noOfDataSet = in.readInt(); 

        // now read the data sets
        for (int i=0; i<noOfDataSet; i++) {
           DataSet2D dataSet = null;
 
           sz = in.readInt();
           byteArr = new byte[sz];
           in.read(byteArr, 0, sz);
           title2 = new String(byteArr);
 
           int noOfTimeData = in.readInt(); 
           for (int k=0; k<noOfTimeData; k++) {
              TimeData2D readed = SerialMgr.parse2dStream(in);
 
              if (k == 0) 
                  dataSet = new DataSet2D(title2, readed);
              else 
                  dataSet.addTimeData(readed);
	   }
 
           if (i == 0) {
               long fid = svServer.fmanager.register(dataSet, -1, cvalues);
               gframe = (GFrame2D)svServer.fmanager.getGFrame(fid);
               ret = fid;
           } else
               gframe.addDataSet(dataSet);
        } 

	///////////////////////////////////////////
        // NEED COLLABORATION
        //////////////////////////////////////////
        CollSender sender = new CollSender(svServer.cmanager, gframe);
        sender.start();

        in.close();
        return ret;

     } catch (Exception e) {
        System.out.println("Can't read the 2d data from a file ...");
        return -1;
     }     

  }

  // read 3d file
  public static long read_3d(svserver svServer, DataInputStream in, int mode) {

     String title = null, title2 = null;
     String configs = null;
     String drawings = null;
     GFrame3D gframe = null;
     Config3D cvalues = null;

     byte[] byteArr;
     int sz;

     long ret = -1;

     try {
        // read title 
        sz = in.readInt();
        byteArr = new byte[sz];
        in.read(byteArr, 0, sz);
        title = new String(byteArr);

        // read configuration stream
        sz = in.readInt();
        if (sz > 0) {
           byteArr = new byte[sz];
           in.read(byteArr, 0, sz);
           cvalues = new Config3D(new String(byteArr));
        } else 
           cvalues = null;

        if (mode == 1) 
           cvalues = svServer.configHandler.config3d;
        else {
           Config3D cvalues2 = svServer.configHandler.config3d;
           cvalues = cvalues2;
        }

        // read drawing objects stream
        sz = in.readInt();    // FLUSH OUT SINCE NOT IMPLEMENTED

        // read whole data information
        int wdata = in.readInt(); 

        // read the number of data sets
        int noOfDataSet = in.readInt(); 

        // now read the data sets
        for (int i=0; i<noOfDataSet; i++) {
           DataSet3D dataSet = null;
 
           sz = in.readInt();
           byteArr = new byte[sz];
           in.read(byteArr, 0, sz);
           title2 = new String(byteArr);
 
           int noOfTimeData = in.readInt(); 
           for (int k=0; k<noOfTimeData; k++) {
              TimeData3D readed = SerialMgr.parse3dStream(in);
 
              if (k == 0) {
                  if (wdata != 0 || this_filename != null) 
                     dataSet = new DataSet3D(title2, -1, null, -1, readed, this_filename);
                  else
                     dataSet = new DataSet3D(title2, -1, null, -1, readed);
              } else 
                  dataSet.addTimeData(readed);
	   }
 
           if (i == 0) {
               long fid = svServer.fmanager.register(dataSet, -1, cvalues);
               gframe = (GFrame3D)svServer.fmanager.getGFrame(fid);
               ret = fid;

           } else {
               // NOTHING TO DO SINCE ONLY ONE DATASET PERMITTED
           }
        } 

	///////////////////////////////////////////
        // NEED COLLABORATION
        //////////////////////////////////////////
        CollSender sender = new CollSender(svServer.cmanager, gframe);
        sender.start();

        in.close();
        return ret;

     } catch (Exception e) {
        System.out.println("Can't read the 3d data from a file ...");
e.printStackTrace();
        return -1;
     }     

  }

  // read connection list file
  public static long read_con(svserver svServer, DataInputStream in, int mode) {
     String title = null, title2 = null;
     String configs = null;
     String drawings = null;
     GFPlane gframe = null;
     CPlane cvalues = null;

     byte[] byteArr;
     int sz;

     long ret = -1;

     try {
        // read title 
        sz = in.readInt();
        byteArr = new byte[sz];
        in.read(byteArr, 0, sz);
        title = new String(byteArr);

        // read configuration stream
        sz = in.readInt();
        if (sz > 0) {
           byteArr = new byte[sz];
           in.read(byteArr, 0, sz);
           cvalues = new CPlane(new String(byteArr));
        } else
           cvalues = null;

        if (mode == 1) 
           cvalues = svServer.configHandler.config1d;
        else {
           CPlane cvalues2 = svServer.configHandler.config1d;
           cvalues2.xLabel = cvalues.xLabel;
           cvalues2.yLabel = cvalues.yLabel;
           cvalues = cvalues2;
        }

        // read drawing objects stream
        sz = in.readInt();
        if (sz > 0) {
           byteArr = new byte[sz];
           in.read(byteArr, 0, sz);
           drawings = new String(byteArr);
        } 

        // read the number of data sets
        int noOfDataSet = in.readInt(); 

        // now read the data sets
        for (int i=0; i<noOfDataSet; i++) {
           DataSetCon dataSet = null;
 
           sz = in.readInt();
           byteArr = new byte[sz];
           in.read(byteArr, 0, sz);
           title2 = new String(byteArr);
 
           int noOfTimeData = in.readInt(); 
           for (int k=0; k<noOfTimeData; k++) {
              TimeDataCon readed = SerialMgr.parseconStream(in);
 
              if (k == 0) 
                  dataSet = new DataSetCon(title2, readed);
              else 
                  dataSet.addTimeData(readed);
	   }
 
           if (i == 0) {
               long fid = svServer.fmanager.register(dataSet, -1, cvalues);
               gframe = (GFPlane)svServer.fmanager.getGFrame(fid);
               ret = fid;
           } else
               gframe.addDataSet(dataSet);
        } 
        gframe.createWhiteboard(drawings);
      
	///////////////////////////////////////////
        // NEED COLLABORATION
        //////////////////////////////////////////
        CollSender sender = new CollSender(svServer.cmanager, gframe);
        sender.start();
   
        in.close();
        return ret;

     } catch (Exception e) {
        System.out.println("Can't read the connection list data from a file ...");
        return -1;
     }     

  }

  // read vector file
  public static long read_vec(svserver svServer, DataInputStream in, int mode) {
     String title = null, title2 = null;
     String configs = null;
     String drawings = null;
     GFPlane gframe = null;
     CPlane cvalues = null;

     byte[] byteArr;
     int sz;
     
     long ret = -1;

     try {
        // read title 
        sz = in.readInt();
        byteArr = new byte[sz];
        in.read(byteArr, 0, sz);
        title = new String(byteArr);

        // read configuration stream
        sz = in.readInt();
        if (sz > 0) {
           byteArr = new byte[sz];
           in.read(byteArr, 0, sz);
           cvalues = new CPlane(new String(byteArr));
        } else
           cvalues = null;

        if (mode == 1) 
           cvalues = svServer.configHandler.config1d;
        else {
           CPlane cvalues2 = svServer.configHandler.config1d;
           cvalues2.xLabel = cvalues.xLabel;
           cvalues2.yLabel = cvalues.yLabel;
           cvalues = cvalues2;
        }


        // read drawing objects stream
        sz = in.readInt();
        if (sz > 0) {
           byteArr = new byte[sz];
           in.read(byteArr, 0, sz);
           drawings = new String(byteArr);
        } 

        // read the number of data sets
        int noOfDataSet = in.readInt(); 

        // now read the data sets
        for (int i=0; i<noOfDataSet; i++) {
           DataSetVec dataSet = null;
 
           sz = in.readInt();
           byteArr = new byte[sz];
           in.read(byteArr, 0, sz);
           title2 = new String(byteArr);
 
           int noOfTimeData = in.readInt(); 
           for (int k=0; k<noOfTimeData; k++) {
              TimeDataVec readed = SerialMgr.parsevecStream(in);
 
              if (k == 0) 
                  dataSet = new DataSetVec(title2, readed);
              else 
                  dataSet.addTimeData(readed);
	   }
 
           if (i == 0) {
               long fid = svServer.fmanager.register(dataSet, -1, cvalues);
               gframe = (GFPlane)svServer.fmanager.getGFrame(fid);
               ret = fid;
           } else
               gframe.addDataSet(dataSet);
        } 
        gframe.createWhiteboard(drawings);
      
	///////////////////////////////////////////
        // NEED COLLABORATION
        //////////////////////////////////////////
        CollSender sender = new CollSender(svServer.cmanager, gframe);
        sender.start();
   
        in.close();
        return ret;

     } catch (Exception e) {
        System.out.println("Can't read the vector data from a file ...");
        return -1;
     }     

  }


  // read bar chart file
  public static long read_bar(svserver svServer, DataInputStream in, int mode) {
     String title = null, title2 = null;
     String configs = null;
     String drawings = null;
     GFPlane gframe = null;
     CPlane cvalues = null;

     byte[] byteArr;
     int sz;
     
     long ret = -1;

     try {
        // read title 
        sz = in.readInt();
        byteArr = new byte[sz];
        in.read(byteArr, 0, sz);
        title = new String(byteArr);

        // read configuration stream
        sz = in.readInt();
        if (sz > 0) {
           byteArr = new byte[sz];
           in.read(byteArr, 0, sz);
           cvalues = new CPlane(new String(byteArr));
        } else
           cvalues = null;

        if (mode == 1) 
           cvalues = svServer.configHandler.config1d;
        else {
           CPlane cvalues2 = svServer.configHandler.config1d;
           cvalues2.xLabel = cvalues.xLabel;
           cvalues2.yLabel = cvalues.yLabel;
           cvalues = cvalues2;
        }

        // read drawing objects stream
        sz = in.readInt();
        if (sz > 0) {
           byteArr = new byte[sz];
           in.read(byteArr, 0, sz);
           drawings = new String(byteArr);
        } 

        // read the number of data sets
        int noOfDataSet = in.readInt(); 

        // now read the data sets
        for (int i=0; i<noOfDataSet; i++) {
           DataSetBar dataSet = null;
 
           sz = in.readInt();
           byteArr = new byte[sz];
           in.read(byteArr, 0, sz);
           title2 = new String(byteArr);
 
           int noOfTimeData = in.readInt(); 
           for (int k=0; k<noOfTimeData; k++) {
              TimeDataBar readed = SerialMgr.parsebarStream(in);
 
              if (k == 0) 
                  dataSet = new DataSetBar(title2, readed);
              else 
                  dataSet.addTimeData(readed);
	   }
 
           if (i == 0) {
               long fid = svServer.fmanager.register(dataSet, -1, cvalues);
               gframe = (GFPlane)svServer.fmanager.getGFrame(fid);
               ret = fid;
           } else
               gframe.addDataSet(dataSet);
        } 
        gframe.createWhiteboard(drawings);
      
	///////////////////////////////////////////
        // NEED COLLABORATION
        //////////////////////////////////////////
        CollSender sender = new CollSender(svServer.cmanager, gframe);
        sender.start();
   
        in.close();
        return ret;

     } catch (Exception e) {
        System.out.println("Can't read the bar chart data from a file ...");
        return -1;
     }     

  }

  // read tree data file
  public static long read_tree(svserver svServer, DataInputStream in, int mode) {
     String title = null, title2 = null;
     String configs = null;
     String drawings = null;
     GFPlane gframe = null;
     CPlane cvalues = null;

     byte[] byteArr;
     int sz;
     
     long ret = -1;

     try {
        // read title 
        sz = in.readInt();
        byteArr = new byte[sz];
        in.read(byteArr, 0, sz);
        title = new String(byteArr);

        // read configuration stream
        sz = in.readInt();
        if (sz > 0) {
           byteArr = new byte[sz];
           in.read(byteArr, 0, sz);
           cvalues = new CPlane(new String(byteArr));
        } else
           cvalues = null;

        if (mode == 1) 
           cvalues = svServer.configHandler.config1d;
        else {
           CPlane cvalues2 = svServer.configHandler.config1d;
           cvalues2.xLabel = cvalues.xLabel;
           cvalues2.yLabel = cvalues.yLabel;
           cvalues = cvalues2;
        }

        // read drawing objects stream
        sz = in.readInt();
        if (sz > 0) {
           byteArr = new byte[sz];
           in.read(byteArr, 0, sz);
           drawings = new String(byteArr);
        } 

        // read the number of data sets
        int noOfDataSet = in.readInt(); 

        // now read the data sets
        for (int i=0; i<noOfDataSet; i++) {
           DataSetTree dataSet = null;
 
           sz = in.readInt();
           byteArr = new byte[sz];
           in.read(byteArr, 0, sz);
           title2 = new String(byteArr);
 
           int noOfTimeData = in.readInt(); 
           for (int k=0; k<noOfTimeData; k++) {
              TimeDataTree readed = SerialMgr.parsetreeStream(in);
 
              if (k == 0) 
                  dataSet = new DataSetTree(title2, readed);
              else 
                  dataSet.addTimeData(readed);
	   }
 
           if (i == 0) {
               long fid = svServer.fmanager.register(dataSet, -1, cvalues);
               gframe = (GFPlane)svServer.fmanager.getGFrame(fid);
               ret = fid;
           } else
               gframe.addDataSet(dataSet);
        } 
        gframe.createWhiteboard(drawings);
      
	///////////////////////////////////////////
        // NEED COLLABORATION
        //////////////////////////////////////////
        CollSender sender = new CollSender(svServer.cmanager, gframe);
        sender.start();
   
        in.close();
        return ret;

     } catch (Exception e) {
        System.out.println("Can't read the tree data from a file ...");
        return -1;
     }     

  }


  // send 1d file
  public static void send_1d(DataInputStream in, DataOutputStream out) {

     byte[] byteArr;
     int sz;

     try {
        // send command to the server
        out.writeInt(1);

        // send dimnension
        out.writeInt(DataSet.ONED);

        // read title 
        sz = in.readInt();
        out.writeInt(sz);

        byteArr = new byte[sz];
        in.read(byteArr, 0, sz);
        out.write(byteArr, 0, sz);

        // read configuration stream
        sz = in.readInt();

        if (sz > 0) {
           byteArr = new byte[sz];
           in.read(byteArr, 0, sz);
           out.writeInt(sz);
           out.write(byteArr, 0, sz);
        } else
           out.writeInt(0);

        // read drawing objects stream
        sz = in.readInt();
        if (sz > 0) {
           byteArr = new byte[sz];
           in.read(byteArr, 0, sz);
           out.writeInt(sz);
           out.write(byteArr, 0, sz);
        } else
           out.writeInt(0);

        // read the number of data sets
        int noOfDataSet = in.readInt(); 
        out.writeInt(noOfDataSet);

        // now read the data sets
        for (int i=0; i<noOfDataSet; i++) {
           sz = in.readInt();     // read dataset title
           out.writeInt(sz);

           byteArr = new byte[sz];
           in.read(byteArr, 0, sz);
           out.write(byteArr, 0, sz);
 
           int noOfTimeData = in.readInt();
           out.writeInt(noOfTimeData);

           for (int k=0; k<noOfTimeData; k++) {
              TimeData1D readed = SerialMgr.parse1dStream(in);
              readed.sendingData(out);              
	   }
        } 

        in.close();
        out.close();
     } catch (Exception e) {
        System.out.println("Can't read the 1d data from a file ...");
     }     
  }

  // send 2d file
  public static void send_2d(DataInputStream in, DataOutputStream out) {

     byte[] byteArr;
     int sz;

     try {
        // send command to the server
        out.writeInt(1);

        // send dimnension
        out.writeInt(DataSet.TWOD);

        // read title 
        sz = in.readInt();
        out.writeInt(sz);

        byteArr = new byte[sz];
        in.read(byteArr, 0, sz);
        out.write(byteArr, 0, sz);

        // read configuration stream
        sz = in.readInt();

        if (sz > 0) {
           byteArr = new byte[sz];
           in.read(byteArr, 0, sz);
           out.writeInt(sz);
           out.write(byteArr, 0, sz);
        } else
           out.writeInt(0);

        // read drawing objects stream
        sz = in.readInt();
        if (sz > 0) {
           byteArr = new byte[sz];
           in.read(byteArr, 0, sz);
           out.writeInt(sz);
           out.write(byteArr, 0, sz);
        } else
           out.writeInt(0);

        // read the number of data sets
        int noOfDataSet = in.readInt(); 
        out.writeInt(noOfDataSet);

        // now read the data sets
        for (int i=0; i<noOfDataSet; i++) {
           sz = in.readInt();     // read dataset title
           out.writeInt(sz);

           byteArr = new byte[sz];
           in.read(byteArr, 0, sz);
           out.write(byteArr, 0, sz);
 
           int noOfTimeData = in.readInt();
           out.writeInt(noOfTimeData);

           for (int k=0; k<noOfTimeData; k++) {
              TimeData2D readed = SerialMgr.parse2dStream(in);
              readed.sendingData(out);              
	   }
        } 

        in.close();
        out.close();
     } catch (Exception e) {
        System.out.println("Can't read the 2d data from a file ...");
     }     
  }

  // send 3d file
  public static void send_3d(DataInputStream in, DataOutputStream out) {
System.out.println("Not implemented yet ...");
  }

  // send connection list file
  public static void send_con(DataInputStream in, DataOutputStream out) {
System.out.println("Not implemented yet ...");
  }

  // send vector file
  public static void send_vec(DataInputStream in, DataOutputStream out) {
System.out.println("Not implemented yet ...");
  }


  // send 1d ascii file format data to the server
  private static void sendAscii_1d(String filename, 
                      BufferedReader dis, DataOutputStream os) {

     StringTokenizer st;        // to parse the numeric data
     String input;

     Vector xvec = null, yvec = null;
     Vector timeVec = new Vector();  // to hold time data
     Vector nxVec = new Vector();    // to hold number of points
     int nx = 0;

     try {

        while ((input = dis.readLine()) != null) {
           st = new StringTokenizer(input);

           if (st.countTokens() == 1) {
              if (xvec != null) 
                 nxVec.addElement(new Integer(nx));
              else {
                 xvec = new Vector();
                 yvec = new Vector();
              }

              timeVec.addElement(Float.valueOf(st.nextToken()));
              nx = 0;

           } else if (st.countTokens() == 2) {
              xvec.addElement(Float.valueOf(st.nextToken()));
              yvec.addElement(Float.valueOf(st.nextToken()));
              nx++;
           }
        }
        nxVec.addElement(new Integer(nx));

        // now send the data to the server
        
        // first send command 
        os.writeInt(12);      // via java_mser

        // send title information 
        byte[] sentArr = filename.getBytes();
        // send size of dataset array
        os.writeInt(sentArr.length); 
        os.write(sentArr, 0, sentArr.length);

	// send nt
        int nt = timeVec.size();
        os.writeInt(nt);

        // write time data 
        int index = 0;

        for (int i=0; i<nt; i++) {
           os.writeFloat(((Float)timeVec.elementAt(i)).floatValue());
           
           nx = ((Integer)nxVec.elementAt(i)).intValue();
           os.writeInt(nx);

           for (int j=0; j<nx; j++) {
              os.writeFloat(((Float)xvec.elementAt(index)).floatValue());
              os.writeFloat(((Float)yvec.elementAt(index)).floatValue());
              index++;
           }
        }
        os.close();
     } catch (Exception e) {
        System.out.println("Can't continue reading file ...");
        return;
     }
  }


  // send 2d ascii file format data to the server
  private static void sendAscii_2d(String filename, 
                    BufferedReader dis, DataOutputStream os) {
     StringTokenizer st;        // to parse the numeric data
     String input;

     Vector xvec = null, yvec = null, zvec = null;
     Vector timeVec = new Vector();  // to hold time data
     Vector nxVec = new Vector();    // to hold number of points
     int nx = 0;

     try {
        while ((input = dis.readLine()) != null) {
           st = new StringTokenizer(input);

           if (st.countTokens() == 1) {
              if (xvec != null) 
                 nxVec.addElement(new Integer(nx));
              else {
                 xvec = new Vector();
                 yvec = new Vector();
                 zvec = new Vector();
              }

              timeVec.addElement(Float.valueOf(st.nextToken()));
              nx = 0;

           } else if (st.countTokens() == 3) {
              xvec.addElement(Float.valueOf(st.nextToken()));
              yvec.addElement(Float.valueOf(st.nextToken()));
              zvec.addElement(Float.valueOf(st.nextToken()));
              nx++;
           }
        }
        nxVec.addElement(new Integer(nx));

        // now send the data to the server
        
        // first send command 
        os.writeInt(16);      // via java_mser2d

        // send title information 
        byte[] sentArr = filename.getBytes();
        // send size of dataset array
        os.writeInt(sentArr.length); 
        os.write(sentArr, 0, sentArr.length);

	// send nt
        int nt = timeVec.size();
        os.writeInt(nt);

        // write time data 
        int index = 0;

        for (int i=0; i<nt; i++) {
           os.writeFloat(((Float)timeVec.elementAt(i)).floatValue());
           
           nx = ((Integer)nxVec.elementAt(i)).intValue();
           os.writeInt(nx);

           for (int j=0; j<nx; j++) {
              os.writeFloat(((Float)xvec.elementAt(index)).floatValue());
              os.writeFloat(((Float)yvec.elementAt(index)).floatValue());
              os.writeFloat(((Float)zvec.elementAt(index)).floatValue());
              index++;
           }
        }
        os.close();
     } catch (Exception e) {
        System.out.println("Can't continue reading file ...");
     }
  }

  // send 3d ascii file format data to the server
  private static void sendAscii_3d(String filename, 
                    BufferedReader dis, DataOutputStream os) {
System.out.println("(Sending 3d ascii file) Not implemented yet.");
  }

  // send connection list ascii file format data to the server
  private static void sendAscii_con(String filename, 
                    BufferedReader dis, DataOutputStream os) {
System.out.println("(Sending connection list ascii file) Not implemented yet.");
  }


  // send vec(vector) ascii file format data to the server
  private static void sendAscii_vec(String filename, 
                    BufferedReader dis, DataOutputStream os) {
     StringTokenizer st;        // to parse the numeric data
     String input;

     Vector xvec = null, yvec = null, rvec = null, tvec = null;
     Vector timeVec = new Vector();  // to hold time data
     Vector nxVec = new Vector();    // to hold number of points
     int nx = 0;

     try {
        while ((input = dis.readLine()) != null) {
           st = new StringTokenizer(input);

           if (st.countTokens() == 1) {
              if (xvec != null) 
                 nxVec.addElement(new Integer(nx));
              else {
                 xvec = new Vector();
                 yvec = new Vector();
                 rvec = new Vector();
                 tvec = new Vector();
              }

              timeVec.addElement(Float.valueOf(st.nextToken()));
              nx = 0;

           } else if (st.countTokens() == 4) {
              xvec.addElement(Float.valueOf(st.nextToken()));
              yvec.addElement(Float.valueOf(st.nextToken()));
              rvec.addElement(Float.valueOf(st.nextToken()));
              tvec.addElement(Float.valueOf(st.nextToken()));
              nx++;
           }
        }
        nxVec.addElement(new Integer(nx));

        // now send the data to the server
        
        // first send command 
        os.writeInt(27);      // via java_vmser

        // send title information 
        byte[] sentArr = filename.getBytes();
        // send size of dataset array
        os.writeInt(sentArr.length); 
        os.write(sentArr, 0, sentArr.length);

	// send nt
        int nt = timeVec.size();
        os.writeInt(nt);

        // write time data 
        int index = 0;

        for (int i=0; i<nt; i++) {
           os.writeFloat(((Float)timeVec.elementAt(i)).floatValue());
           
           nx = ((Integer)nxVec.elementAt(i)).intValue();
           os.writeInt(nx);

           for (int j=0; j<nx; j++) {
              os.writeFloat(((Float)xvec.elementAt(index)).floatValue());
              os.writeFloat(((Float)yvec.elementAt(index)).floatValue());
              os.writeFloat(((Float)rvec.elementAt(index)).floatValue());
              os.writeFloat(((Float)tvec.elementAt(index)).floatValue());
              index++;
           }
        }
        os.close();
     } catch (Exception e) {
        System.out.println("Can't continue reading file ...");
     }
  }

} // end of class





